/* 
   $Id: sgaddr_main.c,v 1.4 2001/10/02 07:50:34 kondoh Exp $

   Copyright (C) 2001 AXE,Inc. <sg@axe-inc.co.jp>

   This file is part of sgaddr.

   sgaddr is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   sgaddr is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with sgaddr; see the file COPYING.   If not, write to
   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include <X11/X.h>

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk/gdkkeysyms.h>

#include "sgaddr.h"

#include "config.h"

GtkWidget  *notebook;
GtkWidget  *list, *list_label;
  
GtkWidget  *name_entry, *ruby_entry, *home_entry;
GtkWidget  *phone_entry, *mobile_entry, *email_entry, *etc_entry;

GtkWidget  *main_note;
GtkWidget  *main_text;

GtkWidget  *text;

int    array_tail = 0, current_num = 0;
struct person_data *array[MAX_PERSONS]; 

int kana_array[12]; 

static GnomeUIInfo file_menu[] = {
  {
    GNOME_APP_UI_ITEM,
    N_("New"), NULL,
    GTK_SIGNAL_FUNC(press_new_btn), NULL, NULL,
    GNOME_APP_PIXMAP_NONE , NULL,
    0, GDK_CONTROL_MASK, NULL
  },
  {
    GNOME_APP_UI_ITEM,
    N_("Add"), NULL,
    GTK_SIGNAL_FUNC(press_add_btn), NULL, NULL,
    GNOME_APP_PIXMAP_NONE , NULL,
    0, GDK_CONTROL_MASK, NULL
  },
  {
    GNOME_APP_UI_ITEM,
    N_("Del"), NULL,
    GTK_SIGNAL_FUNC(press_del_btn), NULL, NULL,
    GNOME_APP_PIXMAP_NONE , NULL,
    0, GDK_CONTROL_MASK, NULL
  },
  {
    GNOME_APP_UI_ITEM,
    N_("Quit"), NULL,
    GTK_SIGNAL_FUNC(gtk_main_quit), NULL, NULL,
    GNOME_APP_PIXMAP_STOCK , GNOME_STOCK_MENU_QUIT,
    0, GDK_CONTROL_MASK, NULL
  },
  GNOMEUIINFO_END
};

static GnomeUIInfo edit_menu[] = {
  {
    GNOME_APP_UI_ITEM,
    N_("Reedit"), NULL,
    GTK_SIGNAL_FUNC(press_reedit_btn), NULL, NULL,
    GNOME_APP_PIXMAP_NONE , NULL,
    0, GDK_CONTROL_MASK, NULL
  },
  {
    GNOME_APP_UI_ITEM,
    N_("Apply"), NULL,
    GTK_SIGNAL_FUNC(press_apply_btn), NULL, NULL,
    GNOME_APP_PIXMAP_NONE , NULL,
    0, GDK_CONTROL_MASK, NULL
  },
  GNOMEUIINFO_END
};

#if 0
static GnomeUIInfo help_menu[] = {
  GNOMEUIINFO_MENU_ABOUT_ITEM(NULL, NULL), GNOMEUIINFO_END
};
#endif

static GnomeUIInfo main_menu[] = {
  GNOMEUIINFO_SUBTREE(N_("_File"),file_menu),
  GNOMEUIINFO_SUBTREE(N_("_Edit"),edit_menu),

#if 0
  GNOMEUIINFO_MENU_HELP_TREE(help_menu),
#endif

  GNOMEUIINFO_END
};

void set_person_data_to_clist(GtkCList *clist, struct person_data *data)
{
  char *strs[16];

  if(!data) return;

  strs[0] = data->name;
  strs[1] = data->ruby;
  strs[2] = NULL;

  gtk_clist_append(clist, strs);
}

void
set_person_data_all(GtkCList *clist)
{
  int i;

  gtk_clist_clear(clist);

  for(i = 0; i < array_tail; i++)
    set_person_data_to_clist(clist, get_person_data(i));
}

void set_entry_editable(int boolean)
{
  gtk_entry_set_editable(GTK_ENTRY(name_entry),   boolean);
  gtk_entry_set_editable(GTK_ENTRY(ruby_entry),   boolean);
  gtk_entry_set_editable(GTK_ENTRY(home_entry),   boolean);
  gtk_entry_set_editable(GTK_ENTRY(phone_entry),  boolean);
  gtk_entry_set_editable(GTK_ENTRY(mobile_entry), boolean);
  gtk_entry_set_editable(GTK_ENTRY(email_entry),  boolean);
  gtk_entry_set_editable(GTK_ENTRY(etc_entry),    boolean);
}

void kana_button_press(GtkWidget *widget, gpointer data)
{
  int i = (int)data;

  if(kana_array[i] >= 0){
    current_num = kana_array[i];
    gtk_clist_moveto (GTK_CLIST(list), current_num, 0, 0.0, 0.0);
  }
}

void set_string_to_entry(GtkWidget *widget, char *str)
{
  if(str) gtk_entry_set_text(GTK_ENTRY(widget), str);
  else    gtk_editable_delete_text(GTK_EDITABLE(widget), 0, -1);
}

char *
get_string_from_entry(GtkWidget *widget, char *dest)
{
  char *str = NULL;

  str = gtk_entry_get_text(GTK_ENTRY(widget));

  if(str && (strlen(str) > 0)){
    if(dest){
      if(strcmp(str, dest) != 0){
	free(dest);
	dest = (char *)strdup(str);
      }
    }else{
      dest = (char *)strdup(str);
    }
  }else{
    if(dest) free(dest);
    dest = NULL;
  }

  return dest;
}

void
set_person_data_to_entry(struct person_data *data)
{
  set_string_to_entry(name_entry,   data->name);
  set_string_to_entry(ruby_entry,   data->ruby);
  set_string_to_entry(home_entry,   data->home);
  set_string_to_entry(phone_entry,  data->phone);
  set_string_to_entry(mobile_entry, data->mobile);
  set_string_to_entry(email_entry,  data->email);
  set_string_to_entry(etc_entry,    data->etc);
}

void 
select_row(GtkWidget *list, gint row, gint column, GdkEvent *event, gpointer data)
{
  set_person_data_to_entry(get_person_data(row));
  current_num = row;
  gtk_notebook_set_page(GTK_NOTEBOOK(notebook), 1);
}

void quit(GtkWidget *widget, gpointer data)
{
  gtk_main_quit();
  save_person_data();
}

void quit_ev(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	quit(widget, data);
}

void key_press(GtkWidget *widget, GdkEventKey *event, gpointer data)
{
  switch(event->keyval){
  case GDK_Return:
  case GDK_Tab:
    break;

  default:
    break;
  }
}

void switch_page(GtkNotebook *notebook, GtkNotebookPage *page, guint page_num)
{
  if(page_num == 1){
    set_edit_state(STATE_REEDIT);
    gtk_notebook_set_page(GTK_NOTEBOOK(main_note), 0);
  }else if(page_num == 0){
    
  }
}


GtkWidget *
create_entry(GtkWidget *table, char *str, int num, int x, int y)
{
  GtkWidget *button, *entry;

  button = gtk_button_new_with_label(str);
  entry = gtk_entry_new();

  gtk_table_attach(GTK_TABLE(table), button, x, x + 1, y, y + 1,
		   GTK_FILL|GTK_EXPAND, GTK_SHRINK, 0, 0);

  gtk_table_attach(GTK_TABLE(table), entry, x + 1, x + 2, y, y + 1,
		   GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 0, 0);

  gtk_signal_connect(GTK_OBJECT(button), "clicked",
		     GTK_SIGNAL_FUNC(sgaddr_main_button_press), (gpointer) num);

  gtk_signal_connect(GTK_OBJECT(entry), "key_press_event",
		     GTK_SIGNAL_FUNC(key_press), (gpointer) num);

  gtk_widget_show(button);
  gtk_widget_show(entry);

  return entry;
}

GtkWidget *
create_button(GtkWidget *container, char *label, GtkSignalFunc func, int num, int x, int y)
{
  GtkWidget *btn;

  btn = gtk_button_new_with_label(label);
  gtk_signal_connect(GTK_OBJECT(btn), "clicked",
		     GTK_SIGNAL_FUNC(func), (gpointer)num);

  gtk_table_attach(GTK_TABLE(container), btn,
		   x, x + 1, y, y + 1,
		   GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_SHRINK, 0, 0);
  
  gtk_widget_show(btn);

  return btn;
}

int sgaddr_locale_ja_chk(char *locale)
{
#ifdef __FreeBSD__
    return strcmp(locale, "ja_JP.EUC") == 0;
#else
    return strncmp(locale, "ja", 2) == 0;
#endif
}

static void sgaddr_create_list_page()
{
  GtkWidget *list_table;
  GtkWidget *vscroll;
  GtkObject *adjustment;
  
  GtkWidget *kana_table;

  char *list_titles[2];
  char *locale;

  list_titles[0]  = _("Name");
  list_titles[1] = _("Ruby");

  list_label = gtk_label_new(_("List"));
  list_table = gtk_table_new(2, 2, FALSE);
  
  list = gtk_clist_new_with_titles(2, list_titles);
  gtk_signal_connect (GTK_OBJECT (list), "select_row", 
		      GTK_SIGNAL_FUNC (select_row), NULL);

  gtk_clist_column_titles_passive(GTK_CLIST(list));
  gtk_clist_set_column_width(GTK_CLIST(list), 0, 80);
  gtk_clist_set_column_width(GTK_CLIST(list), 1, 80);
  gtk_widget_set_usize(list, 1, 1);

  adjustment = gtk_adjustment_new (0.0, 0.0, 1000000.0, 1.0, 25.0, 25.0);
  gtk_clist_set_vadjustment(GTK_CLIST(list), GTK_ADJUSTMENT(adjustment));
  vscroll = gtk_vscrollbar_new(GTK_ADJUSTMENT(adjustment));

  kana_table = gtk_table_new(2, 6, FALSE);

  create_button(kana_table, _("ABC"),  kana_button_press,  0, 0, 0);
  create_button(kana_table, _("DEF"),  kana_button_press,  1, 1, 0);
  create_button(kana_table, _("GHI"),  kana_button_press,  2, 2, 0);
  create_button(kana_table, _("JKL"),  kana_button_press,  3, 3, 0);
  create_button(kana_table, _("MNO"),  kana_button_press,  4, 4, 0);
  create_button(kana_table, _("PQRS"), kana_button_press, 5, 5, 0);
  create_button(kana_table, _("TUV"),  kana_button_press,  6, 0, 1);
  create_button(kana_table, _("WXYZ"), kana_button_press, 7, 1, 1);

  locale = getenv("LC_MESSAGES");
  if(locale == NULL) locale = getenv("LANG");

  if(locale)
    if(sgaddr_locale_ja_chk(locale)){
      create_button(kana_table, _("RA"),    kana_button_press,  8, 2, 1);
      create_button(kana_table, _("WA"),    kana_button_press,  9, 3, 1);
      create_button(kana_table, _("Alpha"), kana_button_press, 10, 4, 1);
    }

  create_button(kana_table, _("Num"), kana_button_press, 11, 5, 1);

  gtk_table_attach(GTK_TABLE(list_table), list, 0, 1, 0, 1,
		   GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 0, 0);
  
  gtk_table_attach(GTK_TABLE(list_table), vscroll, 1, 2, 0, 1,
		   GTK_SHRINK, GTK_FILL|GTK_EXPAND, 0, 0);

  gtk_table_attach(GTK_TABLE(list_table), kana_table, 0, 1, 1, 2,
		   GTK_FILL|GTK_EXPAND, GTK_SHRINK, 0, 0);  

  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), list_table, list_label);

  gtk_widget_show(kana_table);
  gtk_widget_show(list);
  gtk_widget_show(vscroll);
  gtk_widget_show(list_table);
}

static void sgaddr_create_main_page()
{
  GtkWidget *table;
  GtkWidget *buttons;
  GtkWidget *box;

  main_note = gtk_notebook_new();
  gtk_notebook_set_tab_pos(GTK_NOTEBOOK(main_note), GTK_POS_TOP);
  gtk_notebook_set_show_tabs(GTK_NOTEBOOK(main_note), FALSE);

  /* to make entry widgets  */

  box   = gtk_table_new(2, 1, FALSE);
  table = gtk_table_new(8, 2, FALSE);

  gtk_table_attach(GTK_TABLE(box), table, 0, 1, 0, 1,
		   GTK_FILL|GTK_EXPAND|GTK_SHRINK, 
		   GTK_FILL|GTK_EXPAND|GTK_SHRINK, 0, 0);  

  name_entry   = create_entry(table, _("Name"),   ADDR_NAME,   0, 0);
  ruby_entry   = create_entry(table, _("Ruby"),   ADDR_RUBY,   0, 1);
  home_entry   = create_entry(table, _("Home"),   ADDR_HOME,   0, 2);
  phone_entry  = create_entry(table, _("Phone"),  ADDR_PHONE,  0, 3);
  mobile_entry = create_entry(table, _("Mobile"), ADDR_MOBILE, 0, 4);
  email_entry  = create_entry(table, _("E-mail"), ADDR_EMAIL,  0, 5);
  etc_entry    = create_entry(table, _("Etc"),    ADDR_ETC,    0, 6);
  
  buttons =  gtk_table_new(2, 5, FALSE);
  gtk_notebook_append_page(GTK_NOTEBOOK(main_note), box, NULL);

#if 0
  create_button(buttons, _("New"),   button_press, BTN_NEW,    0, 0);
  create_button(buttons, _("Add"),   button_press, BTN_ADD,    1, 0);
  create_button(buttons, _("Del"),   button_press, BTN_DEL,    2, 0);
  create_button(buttons, _("Reedit"),  button_press, BTN_REEDIT, 3, 0);
  create_button(buttons, _("Apply"), button_press, BTN_APPLY,  4, 0);
#endif 

  create_button(buttons, _("Prev"),  button_press, BTN_PREV,   0, 1);
  create_button(buttons, _("Next"),  button_press, BTN_NEXT,   1, 1);

#if 0
  create_button(buttons, _("Quit"),  quit,         BTN_QUIT,   4, 1);
#endif

  gtk_table_attach(GTK_TABLE(box), buttons, 0, 1, 1, 2,
		   GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		   GTK_FILL|GTK_EXPAND|GTK_SHRINK, 0, 0);  

  gtk_widget_show(buttons);

  gtk_widget_show(box);

  gtk_widget_show(table);

  /* to make a text widget  */

  table = gtk_table_new(2, 2, FALSE);
  main_text  = gtk_text_new(NULL, NULL);  

  gtk_table_attach(GTK_TABLE(table), main_text, 0, 1, 0, 1,
		   GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 0, 0);
  create_button(table, _("Return"), sgaddr_return_button, 0, 0, 1);
  gtk_widget_show(main_text);

  gtk_notebook_append_page(GTK_NOTEBOOK(main_note), table, NULL);
  gtk_widget_show(table);

  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), main_note, 
			   gtk_label_new(_("Detail")));

  gtk_widget_show(main_note);
}

int main(int argc, char **argv)
{
  GtkWidget *app;

  bindtextdomain(PACKAGE, LOCALEDIR);
  textdomain(PACKAGE);

  gtk_set_locale();

  gnome_init (PACKAGE, VERSION, argc, argv);
  app = gnome_app_new(PACKAGE, _("sgaddr"));

  gtk_signal_connect (GTK_OBJECT (app), "delete_event",
		      GTK_SIGNAL_FUNC (quit_ev), NULL);

  notebook = gtk_notebook_new();
  gtk_notebook_set_tab_pos(GTK_NOTEBOOK(notebook), GTK_POS_TOP);
  gnome_app_set_contents(GNOME_APP(app), notebook);

  gtk_signal_connect (GTK_OBJECT (notebook), "switch_page",
		      GTK_SIGNAL_FUNC (switch_page), NULL);
  sgaddr_create_list_page();
  sgaddr_create_main_page();

  gnome_app_create_menus(GNOME_APP(app), main_menu);

  gtk_widget_show(notebook);
  gtk_widget_show(app);

  load_person_data();
  sort_by_ruby();
  set_person_data_all(GTK_CLIST(list));

  if(get_person_data(current_num))
    set_person_data_to_entry(get_person_data(current_num));

  set_entry_editable(FALSE);

  gtk_main();

  save_person_data();
  return 1;
}
